@@ -58,7 +58,11 @@ struct _GstDtsDec {
   sample_t 	 bias;
   gboolean 	 dynamic_range_compression;
   sample_t 	*samples;
+#ifndef DTS_OLD
+  dca_state_t   *state;
+#else
   dts_state_t 	*state;
+#endif
   gboolean       dvdmode;
 
   /* Data left over from the previous buffer */
