@@ -21,11 +21,34 @@
 #include "config.h"
 #endif
 
+#include "gstjackaudiosrc.h"
 #include "gstjackaudiosink.h"
 
+GType
+gst_jack_connect_get_type (void)
+{
+  static GType jack_connect_type = 0;
+  static const GEnumValue jack_connect[] = {
+    {GST_JACK_CONNECT_NONE,
+        "Don't automatically connect ports to physical ports", "none"},
+    {GST_JACK_CONNECT_AUTO,
+        "Automatically connect ports to physical ports", "auto"},
+    {0, NULL, NULL},
+  };
+
+  if (!jack_connect_type) {
+    jack_connect_type = g_enum_register_static ("GstJackConnect", jack_connect);
+  }
+  return jack_connect_type;
+}
+
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  if (!gst_element_register (plugin, "jackaudiosrc", GST_RANK_PRIMARY,
+          GST_TYPE_JACK_AUDIO_SRC))
+    return FALSE;
   if (!gst_element_register (plugin, "jackaudiosink", GST_RANK_PRIMARY,
           GST_TYPE_JACK_AUDIO_SINK))
     return FALSE;
