@@ -48,11 +48,18 @@ enum
   ARG_0,
 };
 
+/* FIXME 0.11: Use a single caps name for jpeg2000 codestreams
+ * and drop the "boxed" variant
+ */
+
 static GstStaticPadTemplate gst_jasper_dec_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("image/x-j2c, "
+        "framerate = " GST_VIDEO_FPS_RANGE ", "
+        "fields = (int) 1; "
+        "image/x-jpc, "
         "framerate = " GST_VIDEO_FPS_RANGE ", "
         "fields = (int) 1; " "image/jp2")
     );
@@ -62,11 +69,9 @@ static GstStaticPadTemplate gst_jasper_dec_src_template =
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_BGR "; "
-        GST_VIDEO_CAPS_RGBA "; " GST_VIDEO_CAPS_ARGB "; "
-        GST_VIDEO_CAPS_BGRA "; " GST_VIDEO_CAPS_ABGR "; "
         GST_VIDEO_CAPS_RGBx "; " GST_VIDEO_CAPS_xRGB "; "
         GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_xBGR "; "
-        GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, AYUV, Y41B, Y42B }"))
+        GST_VIDEO_CAPS_YUV ("{ I420, YV12, YUY2, UYVY, Y41B, Y42B }"))
     );
 
 static void gst_jasper_dec_base_init (gpointer g_class);
@@ -116,6 +121,9 @@ gst_jasper_dec_class_init (GstJasperDecClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
+  GST_DEBUG_CATEGORY_INIT (gst_jasper_dec_debug, "jp2kdec", 0,
+      "Jasper JPEG2000 decoder");
+
   gobject_class->set_property = gst_jasper_dec_set_property;
   gobject_class->get_property = gst_jasper_dec_get_property;
 
@@ -140,7 +148,7 @@ gst_jasper_dec_init (GstJasperDec * dec, GstJasperDecClass * klass)
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
   dec->codec_data = NULL;
-  dec->mat = NULL;
+  dec->buf = NULL;
   gst_jasper_dec_reset (dec);
 }
 
@@ -150,9 +158,9 @@ gst_jasper_dec_reset (GstJasperDec * dec)
   if (dec->codec_data)
     gst_buffer_unref (dec->codec_data);
   dec->codec_data = NULL;
-  if (dec->mat)
-    jas_matrix_destroy (dec->mat);
-  dec->mat = NULL;
+  if (dec->buf)
+    g_free (dec->buf);
+  dec->buf = NULL;
   dec->fmt = -1;
   dec->clrspc = JAS_CLRSPC_UNKNOWN;
   dec->format = GST_VIDEO_FORMAT_UNKNOWN;
@@ -180,7 +188,7 @@ gst_jasper_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
     dec->codec_data = NULL;
   }
 
-  if (!strcmp (mimetype, "image/x-j2c")) {
+  if (!strcmp (mimetype, "image/x-j2c") || !strcmp (mimetype, "image/x-jpc")) {
     const GValue *codec_data;
     gint fields;
 
@@ -206,7 +214,11 @@ gst_jasper_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
 
     dec->fmt = jas_image_strtofmt ("jpc");
     /* strip the j2c box stuff it is embedded in */
-    dec->strip = 8;
+    if (!strcmp (mimetype, "image/x-jpc"))
+      dec->strip = 0;
+    else
+      dec->strip = 8;
+
     codec_data = gst_structure_get_value (s, "codec_data");
     if (codec_data) {
       dec->codec_data = gst_value_get_buffer (codec_data);
@@ -234,7 +246,6 @@ gst_jasper_dec_sink_setcaps (GstPad * pad, GstCaps * caps)
 refuse_caps:
   {
     GST_WARNING_OBJECT (dec, "refused caps %" GST_PTR_FORMAT, caps);
-    gst_object_unref (dec);
     return FALSE;
   }
 }
@@ -252,6 +263,8 @@ gst_jasper_dec_negotiate (GstJasperDec * dec, jas_image_t * image)
   height = jas_image_height (image);
   channels = jas_image_numcmpts (image);
 
+  GST_LOG_OBJECT (dec, "%d x %d, %d components", width, height, channels);
+
   /* jp2c bitstream has no real colour space info (kept in container),
    * so decoder may only pretend to know, where it really does not */
   if (!jas_clrspc_isunknown (dec->clrspc)) {
@@ -391,14 +404,9 @@ gst_jasper_dec_negotiate (GstJasperDec * dec, jas_image_t * image)
     dec->image_size = gst_video_format_get_size (dec->format, width, height);
     dec->alpha = gst_video_format_has_alpha (dec->format);
 
-    if (dec->mat)
-      jas_matrix_destroy (dec->mat);
-    dec->mat = jas_matrix_create (1, dec->width);
-    if (!dec->mat) {
-      dec->format = GST_VIDEO_FORMAT_UNKNOWN;
-      GST_DEBUG_OBJECT (dec, "failed to create helper buffer");
-      goto fail_image;
-    }
+    if (dec->buf)
+      g_free (dec->buf);
+    dec->buf = g_new0 (glong, dec->width);
 
     caps = gst_video_format_new_caps (dec->format, dec->width, dec->height,
         dec->framerate_numerator, dec->framerate_denominator, 1, 1);
@@ -462,6 +470,7 @@ gst_jasper_dec_get_picture (GstJasperDec * dec, guint8 * data,
   for (i = 0; i < dec->channels; ++i) {
     gint x, y, cwidth, cheight, inc, stride, cmpt;
     guint8 *row_pix, *out_pix;
+    glong *tb;
 
     inc = dec->inc[i];
     stride = dec->stride[i];
@@ -469,7 +478,7 @@ gst_jasper_dec_get_picture (GstJasperDec * dec, guint8 * data,
     cheight = dec->cheight[cmpt];
     cwidth = dec->cwidth[cmpt];
 
-    GST_DEBUG_OBJECT (dec,
+    GST_LOG_OBJECT (dec,
         "retrieve component %d<=%d, size %dx%d, offset %d, inc %d, stride %d",
         i, cmpt, cwidth, cheight, dec->offset[i], inc, stride);
 
@@ -477,10 +486,12 @@ gst_jasper_dec_get_picture (GstJasperDec * dec, guint8 * data,
 
     for (y = 0; y < cheight; y++) {
       row_pix = out_pix;
-      if (jas_image_readcmpt (image, i, 0, y, cwidth, 1, dec->mat))
+      tb = dec->buf;
+      if (jas_image_readcmpt2 (image, i, 0, y, cwidth, 1, dec->buf))
         goto fail_image;
       for (x = 0; x < cwidth; x++) {
-        *out_pix = (guint8) jas_matrix_get (dec->mat, 0, x);
+        *out_pix = *tb;
+        tb++;
         out_pix += inc;
       }
       out_pix = row_pix + stride;
@@ -678,31 +689,3 @@ fail_init:
     return GST_STATE_CHANGE_FAILURE;
   }
 }
-
-/* entry point to initialize the plug-in
- * initialize the plug-in itself
- * register the element factories and pad templates
- * register the features
- */
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (gst_jasper_dec_debug, "jp2kdec", 0,
-      "Jasper JPEG2000 decoder");
-
-  if (!gst_element_register (plugin, "jp2kdec", GST_RANK_MARGINAL,
-          GST_TYPE_JASPER_DEC))
-    return FALSE;
-
-  /* plugin initialisation succeeded */
-  return TRUE;
-}
-
-
-/* this is the structure that gst-register looks for
- * so keep the name plugin_desc, or you cannot get your plug-in registered */
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "jp2k",
-    "Jasper-based JPEG2000 image decoder",
-    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
