@@ -602,6 +602,10 @@ gst_base_metadata_parse (GstBaseMetadata * filter, const guint8 * buf,
       GST_ELEMENT_ERROR (filter, STREAM, TYPE_NOT_FOUND, (NULL),
           ("Only jpeg and png are supported"));
       goto done;
+    } else {
+      GST_ELEMENT_ERROR (filter, STREAM, FAILED, (NULL),
+          ("Failed to parse stream."));
+      goto done;
     }
   } else if (ret == META_PARSING_NEED_MORE_DATA) {
     filter->need_more_data = TRUE;
@@ -1143,6 +1147,7 @@ gst_base_metadata_calculate_offsets (GstBaseMetadata * base)
 
   if (base->state != MT_STATE_PARSED) {
     ret = FALSE;
+    GST_ELEMENT_ERROR (base, STREAM, FAILED, (NULL), ("Invalid state."));
     goto done;
   }
 
@@ -1807,16 +1812,24 @@ gst_base_metadata_sink_activate (GstPad * pad)
   }
 
   if (ret) {
-    gst_pad_activate_pull (pad, FALSE);
-    gst_pad_activate_push (filter->srcpad, FALSE);
-    if (!gst_pad_is_active (pad)) {
-      ret = gst_pad_activate_push (filter->srcpad, TRUE);
-      ret = ret && gst_pad_activate_push (pad, TRUE);
+    GstActivateMode mode;
+
+    /* in gst_base_metadata_pull_range_parse() we could have triggered
+     * negotiation and plugged new downstream elements.
+     * If GST_PAD_ACTIVATE_MODE (filter->srcpad) is GST_ACTIVATE_PULL it means
+     * that downstream is active in pull mode so we don't deactivate pull mode.
+     */
+    GST_OBJECT_LOCK (filter->srcpad);
+    mode = GST_PAD_ACTIVATE_MODE (filter->srcpad);
+    GST_OBJECT_UNLOCK (filter->srcpad);
+
+    if (mode != GST_ACTIVATE_PULL) {
+      /* change from PULL to PUSH */
+      gst_pad_activate_push (pad, TRUE);
     }
   }
 
   return ret;
-
 }
 
 static gboolean
