@@ -204,7 +204,9 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-h264")
+    GST_STATIC_CAPS ("video/x-h264, "
+        "framerate = (fraction) [0/1, MAX], "
+        "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
 
 static void gst_x264_enc_finalize (GObject * object);
@@ -332,7 +334,7 @@ gst_x264_enc_class_init (GstX264EncClass * klass)
           "Enable trellis searched quantization", ARG_TRELLIS_DEFAULT,
           G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_KEYINT_MAX,
-      g_param_spec_uint ("key_int_max", "Key-frame maximal interval",
+      g_param_spec_uint ("key-int-max", "Key-frame maximal interval",
           "Maximal distance between two key-frames (0 for automatic)",
           0, G_MAXINT, ARG_KEYINT_MAX_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_CABAC,
@@ -541,7 +543,12 @@ gst_x264_enc_init_encoder (GstX264Enc * encoder)
   encoder->x264param.i_frame_reference = encoder->ref;
   encoder->x264param.i_bframe = encoder->bframes;
   encoder->x264param.b_bframe_pyramid = encoder->b_pyramid;
+#if X264_BUILD < 63
   encoder->x264param.b_bframe_adaptive = encoder->b_adapt;
+#else
+  encoder->x264param.i_bframe_adaptive =
+      encoder->b_adapt ? X264_B_ADAPT_FAST : X264_B_ADAPT_NONE;
+#endif
   encoder->x264param.b_interlaced = encoder->interlaced;
   encoder->x264param.b_deblocking_filter = 1;
   encoder->x264param.i_deblocking_filter_alphac0 = 0;
@@ -610,7 +617,7 @@ gst_x264_enc_init_encoder (GstX264Enc * encoder)
   encoder->x264enc = x264_encoder_open (&encoder->x264param);
   if (!encoder->x264enc) {
     GST_ELEMENT_ERROR (encoder, STREAM, ENCODE,
-        ("Can not initialize x264 encoder."), (""));
+        ("Can not initialize x264 encoder."), (NULL));
     return FALSE;
   }
 
@@ -660,8 +667,9 @@ gst_x264_enc_header_buf (GstX264Enc * encoder)
 
   /* x264 is expected to return an SEI (some identification info),
    * followed by an SPS and PPS */
-  if (i_nal != 3 || nal[1].i_type != 7 || nal[2].i_type != 8) {
-    GST_ELEMENT_ERROR (encoder, STREAM, ENCODE, NULL,
+  if (i_nal != 3 || nal[1].i_type != 7 || nal[2].i_type != 8 ||
+      nal[1].i_payload < 4 || nal[2].i_payload < 1) {
+    GST_ELEMENT_ERROR (encoder, STREAM, ENCODE, (NULL),
         ("Unexpected x264 header."));
     return NULL;
   }
@@ -707,15 +715,14 @@ gst_x264_enc_header_buf (GstX264Enc * encoder)
 static gboolean
 gst_x264_enc_set_src_caps (GstX264Enc * encoder, GstPad * pad, GstCaps * caps)
 {
-  GstStructure *structure;
   GstBuffer *buf;
   GstCaps *outcaps;
   gboolean res;
 
-  structure = gst_caps_get_structure (caps, 0);
-  structure = gst_structure_copy (structure);
-  gst_structure_set_name (structure, "video/x-h264");
-  outcaps = gst_caps_new_full (structure, NULL);
+  outcaps = gst_caps_new_simple ("video/x-h264",
+      "width", G_TYPE_INT, encoder->width,
+      "height", G_TYPE_INT, encoder->height,
+      "framerate", GST_TYPE_FRACTION, encoder->fps_num, encoder->fps_den, NULL);
 
   if (!encoder->byte_stream) {
     buf = gst_x264_enc_header_buf (encoder);
@@ -935,7 +942,7 @@ gst_x264_enc_encode_frame (GstX264Enc * encoder, x264_picture_t * pic_in,
     duration = GST_BUFFER_DURATION (in_buf);
     gst_buffer_unref (in_buf);
   } else {
-    GST_ELEMENT_ERROR (encoder, STREAM, ENCODE, NULL,
+    GST_ELEMENT_ERROR (encoder, STREAM, ENCODE, (NULL),
         ("Timestamp queue empty."));
     return GST_FLOW_ERROR;
   }
