@@ -39,6 +39,7 @@ static gchar *convert_to_utf8 (const gchar * text, gint length, guint start,
 static gchar *get_encoding (const gchar * text, guint * start_text,
     gboolean * is_multibyte);
 static gchar *get_encoding_and_convert (const gchar * text, guint length);
+static void mpegts_packetizer_destroy_streams_value (gpointer data);
 
 #define CONTINUITY_UNSET 255
 #define MAX_CONTINUITY 15
@@ -95,6 +96,15 @@ mpegts_packetizer_stream_free (MpegTSPacketizerStream * stream)
   g_free (stream);
 }
 
+static void
+mpegts_packetizer_destroy_streams_value (gpointer data)
+{
+  MpegTSPacketizerStream *stream;
+
+  stream = (MpegTSPacketizerStream *) data;
+  mpegts_packetizer_stream_free (stream);
+}
+
 static void
 mpegts_packetizer_clear_section (MpegTSPacketizer * packetizer,
     MpegTSPacketizerStream * stream)
@@ -120,7 +130,8 @@ static void
 mpegts_packetizer_init (MpegTSPacketizer * packetizer)
 {
   packetizer->adapter = gst_adapter_new ();
-  packetizer->streams = g_hash_table_new (g_direct_hash, g_direct_equal);
+  packetizer->streams = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+      NULL, mpegts_packetizer_destroy_streams_value);
 }
 
 static void
@@ -138,24 +149,11 @@ mpegts_packetizer_dispose (GObject * object)
     G_OBJECT_CLASS (mpegts_packetizer_parent_class)->dispose (object);
 }
 
-static gboolean
-stream_foreach_remove (gpointer key, gpointer value, gpointer data)
-{
-  MpegTSPacketizerStream *stream;
-
-  stream = (MpegTSPacketizerStream *) value;
-  mpegts_packetizer_stream_free (stream);
-
-  return TRUE;
-}
-
 static void
 mpegts_packetizer_finalize (GObject * object)
 {
   MpegTSPacketizer *packetizer = GST_MPEGTS_PACKETIZER (object);
 
-  g_hash_table_foreach_remove (packetizer->streams,
-      stream_foreach_remove, packetizer);
   g_hash_table_destroy (packetizer->streams);
 
   if (G_OBJECT_CLASS (mpegts_packetizer_parent_class)->finalize)
@@ -888,7 +886,7 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
           default:
             code_rate_lp_str = "reserved";
         }
-        /* guard is 32 if 0, 16 if 1, 8 if 2, 4 if 4 */
+        /* guard is 32 if 0, 16 if 1, 8 if 2, 4 if 3 */
         switch (guard_interval) {
           case 0:
             guard_interval = 32;
@@ -899,7 +897,7 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
           case 2:
             guard_interval = 8;
             break;
-          case 4:
+          case 3:
             guard_interval = 4;
             break;
           default:             /* make it default to 32 */
@@ -1018,6 +1016,11 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
         gst_structure_set (transport, "delivery", GST_TYPE_STRUCTURE,
             delivery_structure, NULL);
       }
+      /* free the temporary delivery structure */
+      if (delivery_structure != NULL) {
+        gst_structure_free (delivery_structure);
+        delivery_structure = NULL;
+      }
       if ((delivery =
               gst_mpeg_descriptor_find (mpegdescriptor,
                   DESC_DTG_LOGICAL_CHANNEL))) {
@@ -1046,6 +1049,78 @@ mpegts_packetizer_parse_nit (MpegTSPacketizer * packetizer,
         gst_structure_set_value (transport, "channels", &channel_numbers);
         g_value_unset (&channel_numbers);
       }
+      if ((delivery =
+              gst_mpeg_descriptor_find (mpegdescriptor,
+                  DESC_DVB_FREQUENCY_LIST))) {
+        guint8 *current_pos = delivery + 2;
+        GValue frequencies = { 0 };
+        guint8 type;
+
+        g_value_init (&frequencies, GST_TYPE_LIST);
+        type = *current_pos & 0x03;
+        current_pos++;
+
+        if (type) {
+          const gchar *fieldname = NULL;
+
+          while (current_pos < delivery + DESC_LENGTH (delivery) - 3) {
+            guint32 freq = 0;
+            guint8 *frequency_bcd = current_pos;
+            GValue frequency = { 0 };
+
+            switch (type) {
+              case 0x01:
+                /* satellite */
+                freq =
+                    10 * ((frequency_bcd[3] & 0x0F) +
+                    10 * ((frequency_bcd[3] & 0xF0) >> 4) +
+                    100 * (frequency_bcd[2] & 0x0F) +
+                    1000 * ((frequency_bcd[2] & 0xF0) >> 4) +
+                    10000 * (frequency_bcd[1] & 0x0F) +
+                    100000 * ((frequency_bcd[1] & 0xF0) >> 4) +
+                    1000000 * (frequency_bcd[0] & 0x0F) +
+                    10000000 * ((frequency_bcd[0] & 0xF0) >> 4));
+                break;
+              case 0x02:
+                /* cable */
+                freq = 100 *
+                    ((frequency_bcd[3] & 0x0F) +
+                    10 * ((frequency_bcd[3] & 0xF0) >> 4) +
+                    100 * (frequency_bcd[2] & 0x0F) +
+                    1000 * ((frequency_bcd[2] & 0xF0) >> 4) +
+                    10000 * (frequency_bcd[1] & 0x0F) +
+                    100000 * ((frequency_bcd[1] & 0xF0) >> 4) +
+                    1000000 * (frequency_bcd[0] & 0x0F) +
+                    10000000 * ((frequency_bcd[0] & 0xF0) >> 4));
+                break;
+              case 0x03:
+                /* terrestrial */
+                freq = GST_READ_UINT32_BE (current_pos) * 10;
+                break;
+            }
+            g_value_init (&frequency, G_TYPE_UINT);
+            g_value_set_uint (&frequency, freq);
+            gst_value_list_append_value (&frequencies, &frequency);
+            g_value_unset (&frequency);
+            current_pos += 4;
+          }
+
+          switch (type) {
+            case 0x01:
+              fieldname = "frequency-list-satellite";
+              break;
+            case 0x02:
+              fieldname = "frequency-list-cable";
+              break;
+            case 0x03:
+              fieldname = "frequency-list-terrestrial";
+              break;
+          }
+
+          gst_structure_set_value (transport, fieldname, &frequencies);
+          g_value_unset (&frequencies);
+        }
+      }
       gst_mpeg_descriptor_free (mpegdescriptor);
 
       descriptors = g_value_array_new (0);
@@ -1226,10 +1301,11 @@ mpegts_packetizer_parse_sdt (MpegTSPacketizer * packetizer,
               get_encoding_and_convert (serviceprovider_name,
               serviceprovider_name_length);
 
-          gst_structure_set (service, "name", G_TYPE_STRING, servicename_tmp,
-              NULL);
-          gst_structure_set (service, "provider-name", G_TYPE_STRING,
-              serviceprovider_name_tmp, NULL);
+          gst_structure_set (service,
+              "name", G_TYPE_STRING, servicename_tmp,
+              "provider-name", G_TYPE_STRING, serviceprovider_name_tmp,
+              "scrambled", G_TYPE_BOOLEAN, scrambled, NULL);
+
           g_free (servicename_tmp);
           g_free (serviceprovider_name_tmp);
         }
@@ -1468,7 +1544,6 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
         guint8 *extended_descriptor;
         /*GValue extended_items = { 0 }; */
         gchar *extended_text = NULL;
-        gchar *extended_text_tmp;
         /*g_value_init (&extended_items, GST_TYPE_LIST); */
         for (i = 0; i < extended_event_descriptors->len; i++) {
           extended_descriptor = g_array_index (extended_event_descriptors,
@@ -1478,26 +1553,22 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
             if (extended_text) {
               gchar *tmp;
               gchar *old_extended_text = extended_text;
-              tmp = g_strndup ((gchar *)
+              tmp = get_encoding_and_convert ((gchar *)
                   DESC_DVB_EXTENDED_EVENT_text (extended_descriptor),
                   DESC_DVB_EXTENDED_EVENT_text_length (extended_descriptor));
               extended_text = g_strdup_printf ("%s%s", extended_text, tmp);
               g_free (old_extended_text);
               g_free (tmp);
             } else {
-              extended_text = g_strndup ((gchar *)
+              extended_text = get_encoding_and_convert ((gchar *)
                   DESC_DVB_EXTENDED_EVENT_text (extended_descriptor),
                   DESC_DVB_EXTENDED_EVENT_text_length (extended_descriptor));
             }
           }
         }
         if (extended_text) {
-          extended_text_tmp = get_encoding_and_convert (extended_text,
-              strlen (extended_text));
-
           gst_structure_set (event, "extended-text", G_TYPE_STRING,
-              extended_text_tmp, NULL);
-          g_free (extended_text_tmp);
+              extended_text, NULL);
           g_free (extended_text);
         }
         g_array_free (extended_event_descriptors, TRUE);
@@ -1698,6 +1769,7 @@ mpegts_packetizer_parse_eit (MpegTSPacketizer * packetizer,
           }
         }
         gst_structure_set_value (event, "components", &components);
+        g_value_unset (&components);
         g_array_free (component_descriptors, TRUE);
       }
       gst_mpeg_descriptor_free (mpegdescriptor);
@@ -1746,16 +1818,6 @@ error:
   return NULL;
 }
 
-static void
-foreach_stream_clear (gpointer key, gpointer value, gpointer data)
-{
-  MpegTSPacketizerStream *stream = (MpegTSPacketizerStream *) value;
-
-  /* remove the stream */
-  g_object_unref (stream->section_adapter);
-  g_free (stream);
-}
-
 static gboolean
 remove_all (gpointer key, gpointer value, gpointer user_data)
 {
@@ -1765,8 +1827,6 @@ remove_all (gpointer key, gpointer value, gpointer user_data)
 void
 mpegts_packetizer_clear (MpegTSPacketizer * packetizer)
 {
-  g_hash_table_foreach (packetizer->streams, foreach_stream_clear, packetizer);
-
   /* FIXME can't use remove_all because we don't depend on 2.12 yet */
   g_hash_table_foreach_remove (packetizer->streams, remove_all, NULL);
   gst_adapter_clear (packetizer->adapter);
