@@ -250,6 +250,8 @@ mpegts_parse_reset (MpegTSParse * parse)
   /* PAT */
   g_hash_table_insert (parse->psi_pids,
       GINT_TO_POINTER (0), GINT_TO_POINTER (1));
+  if (parse->pat != NULL)
+    gst_structure_free (parse->pat);
   parse->pat = NULL;
   /* pmt pids will be added and removed dynamically */
 
@@ -611,6 +613,7 @@ mpegts_parse_request_new_pad (GstElement * element, GstPadTemplate * template,
   pad = mpegts_parse_create_tspad (parse, name)->pad;
   gst_pad_set_active (pad, TRUE);
   gst_element_add_pad (element, pad);
+  g_free (name);
 
   return pad;
 }
