@@ -103,11 +103,20 @@ static GstStaticPadTemplate mpegtsmux_sink_factory =
     GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
-    GST_STATIC_CAPS ("video/mpeg, mpegversion=(int) { 1, 2, 4 }, "
+    GST_STATIC_CAPS ("video/mpeg, "
+        "mpegversion = (int) { 1, 2, 4 }, "
         "systemstream = (boolean) false; "
         "video/x-dirac;"
-        "video/x-h264;" "audio/mpeg, mpegversion = (int) { 1, 2, 4 }")
-    );
+        "video/x-h264;"
+        "audio/mpeg, "
+        "mpegversion = (int) { 1, 2, 4 };"
+        "audio/x-lpcm, "
+        "width = (int) { 16, 20, 24 }, "
+        "rate = (int) { 48000, 96000 }, "
+        "channels = (int) [ 1, 8 ], "
+        "dynamic_range = (int) [ 0, 255 ], "
+        "emphasis = (boolean) { FALSE, TRUE }, "
+        "mute = (boolean) { FALSE, TRUE }; " "audio/x-ac3;" "audio/x-dts"));
 
 static GstStaticPadTemplate mpegtsmux_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -285,6 +294,19 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data, GstPad * pad)
         ts_data->pid);
     ts_data->stream = tsmux_create_stream (mux->tsmux, TSMUX_ST_VIDEO_DIRAC,
         ts_data->pid);
+  } else if (gst_structure_has_name (s, "audio/x-ac3")) {
+    GST_DEBUG_OBJECT (pad, "Creating AC3 stream with PID 0x%04x", ts_data->pid);
+    ts_data->stream = tsmux_create_stream (mux->tsmux, TSMUX_ST_PS_AUDIO_AC3,
+        ts_data->pid);
+  } else if (gst_structure_has_name (s, "audio/x-dts")) {
+    GST_DEBUG_OBJECT (pad, "Creating DTS stream with PID 0x%04x", ts_data->pid);
+    ts_data->stream = tsmux_create_stream (mux->tsmux, TSMUX_ST_PS_AUDIO_DTS,
+        ts_data->pid);
+  } else if (gst_structure_has_name (s, "audio/x-lpcm")) {
+    GST_DEBUG_OBJECT (pad, "Creating LPCM stream with PID 0x%04x",
+        ts_data->pid);
+    ts_data->stream = tsmux_create_stream (mux->tsmux, TSMUX_ST_PS_AUDIO_LPCM,
+        ts_data->pid);
   } else if (gst_structure_has_name (s, "video/x-h264")) {
     const GValue *value;
     GST_DEBUG_OBJECT (pad, "Creating H264 stream with PID 0x%04x",
@@ -377,6 +399,10 @@ mpegtsmux_create_stream (MpegTsMux * mux, MpegTsPadData * ts_data, GstPad * pad)
   }
 
   if (ts_data->stream != NULL) {
+    gst_structure_get_int (s, "rate", &ts_data->stream->audio_sampling);
+    gst_structure_get_int (s, "channels", &ts_data->stream->audio_channels);
+    gst_structure_get_int (s, "bitrate", &ts_data->stream->audio_bitrate);
+
     tsmux_stream_set_buffer_release_func (ts_data->stream, release_buffer_cb);
     tsmux_program_add_stream (mux->program, ts_data->stream);
 
@@ -418,6 +444,7 @@ static MpegTsPadData *
 mpegtsmux_choose_best_stream (MpegTsMux * mux)
 {
   MpegTsPadData *best = NULL;
+  GstCollectData *c_best = NULL;
   GSList *walk;
 
   for (walk = mux->collect->data; walk != NULL; walk = g_slist_next (walk)) {
@@ -428,7 +455,8 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
       if (ts_data->queued_buf == NULL) {
         GstBuffer *buf;
 
-        ts_data->queued_buf = buf = gst_collect_pads_pop (mux->collect, c_data);
+        ts_data->queued_buf = buf =
+            gst_collect_pads_peek (mux->collect, c_data);
 
         if (buf != NULL) {
           if (ts_data->prepare_func) {
@@ -463,8 +491,10 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
 
           /* Choose a stream we've never seen a timestamp for to ensure
            * we push enough buffers from it to reach a timestamp */
-          if (ts_data->last_ts == GST_CLOCK_TIME_NONE)
+          if (ts_data->last_ts == GST_CLOCK_TIME_NONE) {
             best = ts_data;
+            c_best = c_data;
+          }
         } else {
           ts_data->eos = TRUE;
           continue;
@@ -478,12 +508,17 @@ mpegtsmux_choose_best_stream (MpegTsMux * mux)
             best->last_ts != GST_CLOCK_TIME_NONE &&
             ts_data->last_ts < best->last_ts) {
           best = ts_data;
+          c_best = c_data;
         }
       } else {
         best = ts_data;
+        c_best = c_data;
       }
     }
   }
+  if (c_best) {
+    gst_buffer_unref (gst_collect_pads_pop (mux->collect, c_best));
+  }
 
   return best;
 }
