@@ -37,6 +37,7 @@ typedef enum
   GST_VIDEO_PARSE_FORMAT_YV12,
   GST_VIDEO_PARSE_FORMAT_YUY2,
   GST_VIDEO_PARSE_FORMAT_UYVY,
+  GST_VIDEO_PARSE_FORMAT_v210,
   GST_VIDEO_PARSE_FORMAT_RGB = 10,
   GST_VIDEO_PARSE_FORMAT_GRAY
 } GstVideoParseFormat;
@@ -94,6 +95,7 @@ gst_video_parse_format_get_type (void)
     {GST_VIDEO_PARSE_FORMAT_YV12, "YV12", "YV12"},
     {GST_VIDEO_PARSE_FORMAT_YUY2, "YUY2", "YUY2"},
     {GST_VIDEO_PARSE_FORMAT_UYVY, "UYVY", "UYVY"},
+    {GST_VIDEO_PARSE_FORMAT_v210, "v210", "v210"},
     {GST_VIDEO_PARSE_FORMAT_RGB, "RGB", "RGB"},
     {GST_VIDEO_PARSE_FORMAT_GRAY, "GRAY", "GRAY"},
     {0, NULL, NULL}
@@ -143,7 +145,8 @@ gst_video_parse_base_init (gpointer g_class)
 
   caps =
       gst_caps_from_string (GST_VIDEO_CAPS_YUV
-      ("{ I420, YV12, YUY2, UYVY }") ";" "video/x-raw-rgb; video/x-raw-gray");
+      ("{ I420, YV12, YUY2, UYVY, v210 }") ";"
+      "video/x-raw-rgb; video/x-raw-gray");
 
   gst_raw_parse_class_set_src_pad_template (rp_class, caps);
   gst_raw_parse_class_set_multiple_frames_per_buffer (rp_class, FALSE);
@@ -191,19 +194,19 @@ gst_video_parse_class_init (GstVideoParseClass * klass)
           G_BYTE_ORDER, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_RED_MASK,
       g_param_spec_int ("red-mask", "Red mask",
-          "Red mask of images in raw stream", 0, INT_MAX,
+          "Red mask of images in raw stream", INT_MIN, INT_MAX,
           GST_VIDEO_BYTE1_MASK_24_INT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_GREEN_MASK,
       g_param_spec_int ("green-mask", "Green mask",
-          "Green mask of images in raw stream", 0, INT_MAX,
+          "Green mask of images in raw stream", INT_MIN, INT_MAX,
           GST_VIDEO_BYTE2_MASK_24_INT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_BLUE_MASK,
       g_param_spec_int ("blue-mask", "Blue mask",
-          "Blue mask of images in raw stream", 0, INT_MAX,
+          "Blue mask of images in raw stream", INT_MIN, INT_MAX,
           GST_VIDEO_BYTE3_MASK_24_INT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_ALPHA_MASK,
       g_param_spec_int ("alpha-mask", "Alpha mask",
-          "Alpha mask of images in raw stream", 0, INT_MAX, 0,
+          "Alpha mask of images in raw stream", INT_MIN, INT_MAX, 0,
           G_PARAM_READWRITE));
 }
 
@@ -348,6 +351,8 @@ gst_video_parse_format_to_fourcc (GstVideoParseFormat format)
       return GST_MAKE_FOURCC ('Y', 'U', 'Y', '2');
     case GST_VIDEO_PARSE_FORMAT_UYVY:
       return GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
+    case GST_VIDEO_PARSE_FORMAT_v210:
+      return GST_MAKE_FOURCC ('v', '2', '1', '0');
     default:
       g_assert_not_reached ();
   }
@@ -368,6 +373,8 @@ gst_video_parse_update_frame_size (GstVideoParse * vp)
   } else if (vp->format == GST_VIDEO_PARSE_FORMAT_YUY2
       || vp->format == GST_VIDEO_PARSE_FORMAT_UYVY) {
     framesize = GST_ROUND_UP_4 (vp->width * 2) * vp->height;
+  } else if (vp->format == GST_VIDEO_PARSE_FORMAT_v210) {
+    framesize = ((vp->width + 47) / 48) * 128 * vp->height;
   } else if (vp->format == GST_VIDEO_PARSE_FORMAT_RGB) {
     framesize = GST_ROUND_UP_4 (vp->width * vp->bpp / 8) * vp->height;
   } else {
@@ -393,7 +400,7 @@ gst_video_parse_get_caps (GstRawParse * rp)
         "height", G_TYPE_INT, vp->height,
         "format", GST_TYPE_FOURCC,
         gst_video_parse_format_to_fourcc (vp->format), "framerate",
-        GST_TYPE_FRACTION, fps_n, fps_d, "pixel_aspect_ratio",
+        GST_TYPE_FRACTION, fps_n, fps_d, "pixel-aspect-ratio",
         GST_TYPE_FRACTION, vp->par_n, vp->par_d, NULL);
   } else if (vp->format == GST_VIDEO_PARSE_FORMAT_RGB) {
     caps = gst_caps_new_simple ("video/x-raw-rgb",
@@ -402,7 +409,7 @@ gst_video_parse_get_caps (GstRawParse * rp)
         "bpp", G_TYPE_INT, vp->bpp,
         "depth", G_TYPE_INT, vp->depth,
         "framerate", GST_TYPE_FRACTION, fps_n, fps_d,
-        "pixel_aspect_ratio", GST_TYPE_FRACTION, vp->par_n, vp->par_d,
+        "pixel-aspect-ratio", GST_TYPE_FRACTION, vp->par_n, vp->par_d,
         "red_mask", G_TYPE_INT, vp->red_mask,
         "green_mask", G_TYPE_INT, vp->green_mask,
         "blue_mask", G_TYPE_INT, vp->blue_mask,
@@ -415,7 +422,7 @@ gst_video_parse_get_caps (GstRawParse * rp)
         "bpp", G_TYPE_INT, vp->bpp,
         "depth", G_TYPE_INT, vp->depth,
         "framerate", GST_TYPE_FRACTION, fps_n, fps_d,
-        "pixel_aspect_ratio", GST_TYPE_FRACTION, vp->par_n, vp->par_d, NULL);
+        "pixel-aspect-ratio", GST_TYPE_FRACTION, vp->par_n, vp->par_d, NULL);
   }
   return caps;
 }
