@@ -342,17 +342,24 @@ gst_selector_pad_event (GstPad * pad, GstEvent * event)
   gboolean forward = TRUE;
   GstInputSelector *sel;
   GstSelectorPad *selpad;
+  GstPad *prev_active_sinkpad;
+  GstPad *active_sinkpad;
 
   sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
   selpad = GST_SELECTOR_PAD_CAST (pad);
 
-  /* only forward if we are dealing with the active sinkpad */
-  forward = gst_input_selector_is_active_sinkpad (sel, pad);
+  GST_INPUT_SELECTOR_LOCK (sel);
+  prev_active_sinkpad = sel->active_sinkpad;
+  active_sinkpad = gst_input_selector_activate_sinkpad (sel, pad);
 
-  /* forward all events in select_all mode by default */
-  if (sel->select_all) {
-    forward = TRUE;
-  }
+  /* only forward if we are dealing with the active sinkpad or if select_all
+   * is enabled */
+  if (pad != active_sinkpad && !sel->select_all)
+    forward = FALSE;
+  GST_INPUT_SELECTOR_UNLOCK (sel);
+
+  if (prev_active_sinkpad != active_sinkpad && pad == active_sinkpad)
+    g_object_notify (G_OBJECT (sel), "active-pad");
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
@@ -455,6 +462,7 @@ gst_selector_pad_bufferalloc (GstPad * pad, guint64 offset,
   GstInputSelector *sel;
   GstFlowReturn result;
   GstPad *active_sinkpad;
+  GstPad *prev_active_sinkpad;
   GstSelectorPad *selpad;
 
   sel = GST_INPUT_SELECTOR (gst_pad_get_parent (pad));
@@ -463,6 +471,7 @@ gst_selector_pad_bufferalloc (GstPad * pad, guint64 offset,
   GST_DEBUG_OBJECT (pad, "received alloc");
 
   GST_INPUT_SELECTOR_LOCK (sel);
+  prev_active_sinkpad = sel->active_sinkpad;
   active_sinkpad = gst_input_selector_activate_sinkpad (sel, pad);
 
   if (pad != active_sinkpad)
@@ -470,6 +479,9 @@ gst_selector_pad_bufferalloc (GstPad * pad, guint64 offset,
 
   GST_INPUT_SELECTOR_UNLOCK (sel);
 
+  if (prev_active_sinkpad != active_sinkpad && pad == active_sinkpad)
+    g_object_notify (G_OBJECT (sel), "active-pad");
+
   result = gst_pad_alloc_buffer (sel->srcpad, offset, size, caps, buf);
 
 done:
@@ -518,6 +530,7 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   GstInputSelector *sel;
   GstFlowReturn res;
   GstPad *active_sinkpad;
+  GstPad *prev_active_sinkpad;
   GstSelectorPad *selpad;
   GstClockTime end_time, duration;
   GstSegment *seg;
@@ -534,6 +547,7 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
 
   GST_DEBUG_OBJECT (pad, "getting active pad");
 
+  prev_active_sinkpad = sel->active_sinkpad;
   active_sinkpad = gst_input_selector_activate_sinkpad (sel, pad);
 
   /* update the segment on the srcpad */
@@ -586,6 +600,9 @@ gst_selector_pad_chain (GstPad * pad, GstBuffer * buf)
   }
   GST_INPUT_SELECTOR_UNLOCK (sel);
 
+  if (prev_active_sinkpad != active_sinkpad && pad == active_sinkpad)
+    g_object_notify (G_OBJECT (sel), "active-pad");
+
   if (close_event)
     gst_pad_push_event (sel->srcpad, close_event);
 
@@ -785,8 +802,11 @@ gst_input_selector_class_init (GstInputSelectorClass * klass)
    * If @pad is the same as the current active pad, the element will cancel any
    * previous block without adjusting segments.
    *
-   * Since: 0.10.7 the signal changed from accepting the pad name to the pad
-   * object.
+   * <note><simpara>
+   * the signal changed from accepting the pad name to the pad object.
+   * </simpara></note>
+   *
+   * Since: 0.10.7
    */
   gst_input_selector_signals[SIGNAL_SWITCH] =
       g_signal_new ("switch", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
@@ -912,6 +932,7 @@ gst_input_selector_set_active_pad (GstInputSelector * self,
     /* schedule a last_stop update if one isn't already scheduled, and a
        segment has been pushed before. */
     memcpy (&self->segment, &old->segment, sizeof (self->segment));
+
     GST_DEBUG_OBJECT (self, "setting stop_time to %" G_GINT64_FORMAT,
         stop_time);
     gst_segment_set_stop (&self->segment, stop_time);
@@ -1026,7 +1047,8 @@ gst_input_selector_event (GstPad * pad, GstEvent * event)
     res = gst_pad_push_event (otherpad, event);
 
     gst_object_unref (otherpad);
-  }
+  } else
+    gst_event_unref (event);
   return res;
 }
 
@@ -1180,6 +1202,8 @@ gst_input_selector_activate_sinkpad (GstInputSelector * sel, GstPad * pad)
   if (active_sinkpad == NULL || sel->select_all) {
     /* first pad we get activity on becomes the activated pad by default, if we
      * select all, we also remember the last used pad. */
+    if (sel->active_sinkpad)
+      gst_object_unref (sel->active_sinkpad);
     active_sinkpad = sel->active_sinkpad = gst_object_ref (pad);
     GST_DEBUG_OBJECT (sel, "Activating pad %s:%s", GST_DEBUG_PAD_NAME (pad));
   }
@@ -1239,6 +1263,7 @@ gst_input_selector_release_pad (GstElement * element, GstPad * pad)
   /* if the pad was the active pad, makes us select a new one */
   if (sel->active_sinkpad == pad) {
     GST_DEBUG_OBJECT (sel, "Deactivating pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+    gst_object_unref (sel->active_sinkpad);
     sel->active_sinkpad = NULL;
   }
   sel->n_pads--;
